@@ -76,7 +76,7 @@ mktime_from_string (char *date,
 	fields = g_strsplit (time, ":", 2);
 	if (fields[0] != NULL) {
 		tm.tm_hour = atoi (fields[0]);
-		if (fields[1] != NULL) 
+		if (fields[1] != NULL)
 			tm.tm_min = atoi (fields[1]);
 	}
 	tm.tm_sec = 0;
@@ -87,7 +87,7 @@ mktime_from_string (char *date,
 
 
 static void
-process_line (char     *line, 
+process_line (char     *line,
 	      gpointer  data)
 {
 	FileData      *fdata;
@@ -105,7 +105,7 @@ process_line (char     *line,
 			else
 				ace_comm->command_type = FR_ACE_COMMAND_NONFREE;
 		}
-		return;		
+		return;
 	}
 
 	if (! ace_comm->list_started) {
@@ -143,7 +143,7 @@ process_line (char     *line,
 	if (field_name[0] != '/') {
 		fdata->full_path = g_strconcat ("/", field_name, NULL);
 		fdata->original_path = fdata->full_path + 1;
-	} 
+	}
 	else {
 		fdata->full_path = g_strdup (field_name);
 		fdata->original_path = fdata->full_path;
@@ -156,19 +156,18 @@ process_line (char     *line,
 
 	if (*fdata->name == 0)
 		file_data_free (fdata);
-	else 
+	else
 		fr_command_add_file (comm, fdata);
 }
 
 
 static void
-fr_command_ace_list (FrCommand  *comm,
-		    const char *password)
+fr_command_ace_list (FrCommand  *comm)
 {
 	FR_COMMAND_ACE (comm)->list_started = FALSE;
 	FR_COMMAND_ACE (comm)->command_type = FR_ACE_COMMAND_UNKNOWN;
 
-	fr_process_set_out_line_func (FR_COMMAND (comm)->process, 
+	fr_process_set_out_line_func (FR_COMMAND (comm)->process,
 				      process_line,
 				      comm);
 
@@ -183,18 +182,18 @@ fr_command_ace_list (FrCommand  *comm,
 
 static void
 fr_command_ace_extract (FrCommand   *comm,
+			const char  *from_file,
 			GList       *file_list,
-			const char *dest_dir,
+			const char  *dest_dir,
 			gboolean     overwrite,
 			gboolean     skip_older,
-			gboolean     junk_paths,
-			const char *password)
+			gboolean     junk_paths)
 {
 	GList *scan;
 
 	fr_process_begin_command (comm->process, "unace");
 
-	if (dest_dir != NULL) 
+	if (dest_dir != NULL)
 		fr_process_set_working_dir (comm->process, dest_dir);
 
 	if (junk_paths)
@@ -212,8 +211,7 @@ fr_command_ace_extract (FrCommand   *comm,
 
 
 static void
-fr_command_ace_test (FrCommand   *comm,
-                     const char  *password)
+fr_command_ace_test (FrCommand   *comm)
 {
         fr_process_begin_command (comm->process, "unace");
         fr_process_add_arg (comm->process, "t");
@@ -224,26 +222,38 @@ fr_command_ace_test (FrCommand   *comm,
 
 
 static void
-fr_command_ace_handle_error (FrCommand   *comm, 
+fr_command_ace_handle_error (FrCommand   *comm,
 			     FrProcError *error)
 {
 	/* FIXME */
 }
 
 
-static void
-fr_command_ace_set_mime_type (FrCommand  *comm,
-			      const char *mime_type)
+const char *ace_mime_type[] = { "application/x-ace", NULL };
+
+
+const char **
+fr_command_ace_get_mime_types (FrCommand *comm)
 {
-	FR_COMMAND_CLASS (parent_class)->set_mime_type (comm, mime_type);
-	
-	comm->capabilities |= FR_COMMAND_CAP_ARCHIVE_MANY_FILES;
-	if (is_program_in_path ("unace")) 
-		comm->capabilities |= FR_COMMAND_CAP_READ;
+	return ace_mime_type;
 }
 
 
-static void 
+FrCommandCap
+fr_command_ace_get_capabilities (FrCommand  *comm,
+			         const char *mime_type)
+{
+	FrCommandCap capabilities;
+
+	capabilities = FR_COMMAND_CAN_ARCHIVE_MANY_FILES;
+	if (is_program_in_path ("unace"))
+		capabilities |= FR_COMMAND_CAN_READ;
+
+	return capabilities;
+}
+
+
+static void
 fr_command_ace_class_init (FrCommandAceClass *class)
 {
         GObjectClass   *gobject_class = G_OBJECT_CLASS (class);
@@ -254,15 +264,16 @@ fr_command_ace_class_init (FrCommandAceClass *class)
 
 	gobject_class->finalize = fr_command_ace_finalize;
 
-        afc->list           = fr_command_ace_list;
-	afc->extract        = fr_command_ace_extract;
-	afc->test           = fr_command_ace_test;
-	afc->handle_error   = fr_command_ace_handle_error;
-	afc->set_mime_type  = fr_command_ace_set_mime_type;
+        afc->list             = fr_command_ace_list;
+	afc->extract          = fr_command_ace_extract;
+	afc->test             = fr_command_ace_test;
+	afc->handle_error     = fr_command_ace_handle_error;
+	afc->get_mime_types   = fr_command_ace_get_mime_types;
+	afc->get_capabilities = fr_command_ace_get_capabilities;
 }
 
- 
-static void 
+
+static void
 fr_command_ace_init (FrCommand *comm)
 {
 	comm->propAddCanUpdate             = TRUE;
@@ -275,12 +286,12 @@ fr_command_ace_init (FrCommand *comm)
 }
 
 
-static void 
+static void
 fr_command_ace_finalize (GObject *object)
 {
         g_return_if_fail (object != NULL);
         g_return_if_fail (FR_IS_COMMAND_ACE (object));
-	
+
 	/* Chain up */
         if (G_OBJECT_CLASS (parent_class)->finalize)
 		G_OBJECT_CLASS (parent_class)->finalize (object);
