@@ -49,7 +49,7 @@ mktime_from_string (char *date_s,
 {
 	struct tm   tm = {0, };
 	char      **fields;
-	
+
 	tm.tm_isdst = -1;
 
 	/* date */
@@ -120,7 +120,7 @@ list__process_line (char     *line,
 		if (*name_field == '/') {
 			fdata->full_path = g_strdup (name_field);
 			fdata->original_path = fdata->full_path;
-		} 
+		}
 		else {
 			fdata->full_path = g_strconcat ("/", name_field, NULL);
 			fdata->original_path = fdata->full_path + 1;
@@ -130,7 +130,7 @@ list__process_line (char     *line,
 
 		fdata->name = g_strdup (file_name_from_path (fdata->full_path));
 		fdata->path = remove_level_from_path (fdata->full_path);
-	} 
+	}
 	else if (arj_comm->line_no == 2) { /* Read file size and date. */
 		FileData  *fdata;
 		char     **fields;
@@ -145,7 +145,7 @@ list__process_line (char     *line,
 		if (strcmp (fields[1], "MS-DOS") == 0)
 			fdata->encrypted = (g_ascii_strcasecmp (fields[7], "11") == 0);
 		else
-			fdata->encrypted = (g_ascii_strcasecmp (fields[9], "11") == 0);			
+			fdata->encrypted = (g_ascii_strcasecmp (fields[9], "11") == 0);
 		g_strfreev (fields);
 
 		if (*fdata->name == 0)
@@ -154,14 +154,13 @@ list__process_line (char     *line,
 			fr_command_add_file (comm, fdata);
 		arj_comm->fdata = NULL;
 	}
-		
+
 	arj_comm->line_no++;
 }
 
 
 static void
-fr_command_arj_list (FrCommand  *comm,
-		     const char *password)
+fr_command_arj_list (FrCommand  *comm)
 {
 	fr_process_set_out_line_func (FR_COMMAND (comm)->process,
 				      list__process_line,
@@ -179,12 +178,11 @@ fr_command_arj_list (FrCommand  *comm,
 
 static void
 fr_command_arj_add (FrCommand     *comm,
+		    const char    *from_file,
 		    GList         *file_list,
 		    const char    *base_dir,
 		    gboolean       update,
-		    gboolean       recursive,
-		    const char    *password,
-		    FrCompression  compression)
+		    gboolean       recursive)
 {
 	GList *scan;
 
@@ -198,10 +196,10 @@ fr_command_arj_add (FrCommand     *comm,
 	if (update)
 		fr_process_add_arg (comm->process, "-u");
 
-	if (password != NULL) 
-		fr_process_add_arg_concat (comm->process, "-g/", password, NULL);
+	if (comm->password != NULL)
+		fr_process_add_arg_concat (comm->process, "-g/", comm->password, NULL);
 
-	switch (compression) {
+	switch (comm->compression) {
 	case FR_COMPRESSION_VERY_FAST:
 		fr_process_add_arg (comm->process, "-m3"); break;
 	case FR_COMPRESSION_FAST:
@@ -226,8 +224,9 @@ fr_command_arj_add (FrCommand     *comm,
 
 
 static void
-fr_command_arj_delete (FrCommand *comm,
-		       GList     *file_list)
+fr_command_arj_delete (FrCommand  *comm,
+		       const char *from_file,
+		       GList      *file_list)
 {
 	GList *scan;
 
@@ -248,12 +247,12 @@ fr_command_arj_delete (FrCommand *comm,
 
 static void
 fr_command_arj_extract (FrCommand  *comm,
+			const char *from_file,
 			GList      *file_list,
 			const char *dest_dir,
 			gboolean    overwrite,
 			gboolean    skip_older,
-			gboolean    junk_paths,
-			const char *password)
+			gboolean    junk_paths)
 {
 	GList *scan;
 
@@ -264,7 +263,7 @@ fr_command_arj_extract (FrCommand  *comm,
 	else
 		fr_process_add_arg (comm->process, "x");
 
-	if (dest_dir != NULL) 
+	if (dest_dir != NULL)
 		fr_process_add_arg_concat (comm->process, "-ht/", dest_dir, NULL);
 
 	if (! overwrite)
@@ -273,8 +272,8 @@ fr_command_arj_extract (FrCommand  *comm,
 	if (skip_older)
 		fr_process_add_arg (comm->process, "-u");
 
-	if (password != NULL) 
-		fr_process_add_arg_concat (comm->process, "-g/", password, NULL);
+	if (comm->password != NULL)
+		fr_process_add_arg_concat (comm->process, "-g/", comm->password, NULL);
 	else
  		fr_process_add_arg (comm->process, "-g/");
 
@@ -292,13 +291,12 @@ fr_command_arj_extract (FrCommand  *comm,
 
 
 static void
-fr_command_arj_test (FrCommand   *comm,
-		     const char  *password)
+fr_command_arj_test (FrCommand   *comm)
 {
 	fr_process_begin_command (comm->process, "arj");
 	fr_process_add_arg (comm->process, "t");
-	if (password != NULL) 
-		fr_process_add_arg_concat (comm->process, "-g/", password, NULL);
+	if (comm->password != NULL)
+		fr_process_add_arg_concat (comm->process, "-g/", comm->password, NULL);
 	fr_process_add_arg (comm->process, "-i");
 	fr_process_add_arg (comm->process, "-y");
 	fr_process_add_arg (comm->process, "-");
@@ -320,15 +318,27 @@ fr_command_arj_handle_error (FrCommand   *comm,
 }
 
 
-static void
-fr_command_arj_set_mime_type (FrCommand  *comm,
-		 	      const char *mime_type)
+const char *arj_mime_type[] = { "application/x-arj", NULL };
+
+
+const char **
+fr_command_arj_get_mime_types (FrCommand *comm)
 {
-	FR_COMMAND_CLASS (parent_class)->set_mime_type (comm, mime_type);
-	
-	comm->capabilities |= FR_COMMAND_CAP_ARCHIVE_MANY_FILES;
-	if (is_program_in_path ("arj")) 
-		comm->capabilities |= FR_COMMAND_CAP_READ_WRITE;
+	return arj_mime_type;
+}
+
+
+FrCommandCap
+fr_command_arj_get_capabilities (FrCommand  *comm,
+			         const char *mime_type)
+{
+	FrCommandCap capabilities;
+
+	capabilities = FR_COMMAND_CAN_ARCHIVE_MANY_FILES | FR_COMMAND_CAN_ENCRYPT;
+	if (is_program_in_path ("arj"))
+		capabilities |= FR_COMMAND_CAN_READ_WRITE;
+
+	return capabilities;
 }
 
 
@@ -343,13 +353,14 @@ fr_command_arj_class_init (FrCommandArjClass *class)
 
 	gobject_class->finalize = fr_command_arj_finalize;
 
-	afc->list           = fr_command_arj_list;
-	afc->add            = fr_command_arj_add;
-	afc->delete         = fr_command_arj_delete;
-	afc->extract        = fr_command_arj_extract;
-	afc->test           = fr_command_arj_test;
-	afc->handle_error   = fr_command_arj_handle_error;
-	afc->set_mime_type  = fr_command_arj_set_mime_type;
+	afc->list             = fr_command_arj_list;
+	afc->add              = fr_command_arj_add;
+	afc->delete           = fr_command_arj_delete;
+	afc->extract          = fr_command_arj_extract;
+	afc->test             = fr_command_arj_test;
+	afc->handle_error     = fr_command_arj_handle_error;
+	afc->get_mime_types   = fr_command_arj_get_mime_types;
+	afc->get_capabilities = fr_command_arj_get_capabilities;
 }
 
 
