@@ -128,11 +128,9 @@ list__process_line (char     *line,
 
 
 static void
-fr_command_iso_list (FrCommand  *comm,
-		     const char *password)
+fr_command_iso_list (FrCommand  *comm)
 {
 	FrCommandIso *comm_iso = FR_COMMAND_ISO (comm);
-	char         *e_filename;
 
 	g_free (comm_iso->cur_path);
 	comm_iso->cur_path = NULL;
@@ -141,78 +139,88 @@ fr_command_iso_list (FrCommand  *comm,
 				      list__process_line,
 				      comm);
 
-	e_filename = g_shell_quote (comm->filename);
-
-	fr_process_begin_command (comm->process, "sh " PRIVEXECDIR "isoinfo.sh");
+	fr_process_begin_command (comm->process, "sh");
+	fr_process_add_arg (comm->process, PRIVEXECDIR "isoinfo.sh");
 	fr_process_add_arg (comm->process, "-i");
-	fr_process_add_arg (comm->process, e_filename);
+	fr_process_add_arg (comm->process, comm->filename);
 	fr_process_add_arg (comm->process, "-l");
 	fr_process_end_command (comm->process);
 
-	g_free (e_filename);
-
 	fr_process_start (comm->process);
 }
 
 
 static void
 fr_command_iso_extract (FrCommand  *comm,
+			const char *from_file,
 			GList      *file_list,
 			const char *dest_dir,
 			gboolean    overwrite,
 			gboolean    skip_older,
-			gboolean    junk_paths,
-			const char *password)
+			gboolean    junk_paths)
 {
-	char  *e_dest_dir;
-	char  *e_archive_filename;
 	GList *scan;
 
-	e_dest_dir = fr_command_escape (comm, dest_dir);
-	e_archive_filename = g_shell_quote (comm->filename);
-
 	for (scan = file_list; scan; scan = scan->next) {
 		char       *path = scan->data;
 		const char *filename;
 		char       *file_dir;
-		char       *e_temp_dest_dir = NULL;
 		char       *temp_dest_dir = NULL;
 
 		filename = file_name_from_path (path);
 		file_dir = remove_level_from_path (path);
 		if ((file_dir != NULL) && (strcmp (file_dir, "/") != 0))
-			e_temp_dest_dir = g_build_filename (e_dest_dir, file_dir, NULL);
+			temp_dest_dir = g_build_filename (dest_dir, file_dir, NULL);
 		 else
-			e_temp_dest_dir = g_strdup (e_dest_dir);
+			temp_dest_dir = g_strdup (dest_dir);
 		g_free (file_dir);
 
-		if (e_temp_dest_dir == NULL)
+		if (temp_dest_dir == NULL)
 			continue;
 
-		temp_dest_dir = unescape_str (e_temp_dest_dir);
 		make_directory_tree_from_path (temp_dest_dir, 0700, NULL);
 
-		fr_process_begin_command (comm->process, "sh " PRIVEXECDIR "isoinfo.sh");
+		fr_process_begin_command (comm->process, "sh");
 		fr_process_set_working_dir (comm->process, temp_dest_dir);
+		fr_process_add_arg (comm->process, PRIVEXECDIR "isoinfo.sh");
 		fr_process_add_arg (comm->process, "-i");
-		fr_process_add_arg (comm->process, e_archive_filename);
+		fr_process_add_arg (comm->process, comm->filename);
 		fr_process_add_arg (comm->process, "-x");
 		fr_process_add_arg (comm->process, path);
-		fr_process_add_arg (comm->process, ">");
 		fr_process_add_arg (comm->process, filename);
 		fr_process_end_command (comm->process);
 
-		g_free (e_temp_dest_dir);
 		g_free (temp_dest_dir);
 	}
 
-	g_free (e_dest_dir);
-	g_free (e_archive_filename);
-
 	fr_process_start (comm->process);
 }
 
 
+const char *iso_mime_type[] = { "application/x-cd-image", NULL };
+
+
+const char **
+fr_command_iso_get_mime_types (FrCommand *comm)
+{
+	return iso_mime_type;
+}
+
+
+FrCommandCap
+fr_command_iso_get_capabilities (FrCommand  *comm,
+			         const char *mime_type)
+{
+	FrCommandCap capabilities;
+
+	capabilities = FR_COMMAND_CAN_ARCHIVE_MANY_FILES;
+	if (is_program_in_path ("isoinfo"))
+		capabilities |= FR_COMMAND_CAN_READ;
+
+	return capabilities;
+}
+
+
 static void
 fr_command_iso_class_init (FrCommandIsoClass *class)
 {
@@ -224,8 +232,10 @@ fr_command_iso_class_init (FrCommandIsoClass *class)
 
 	gobject_class->finalize = fr_command_iso_finalize;
 
-	afc->list    = fr_command_iso_list;
-	afc->extract = fr_command_iso_extract;
+	afc->list             = fr_command_iso_list;
+	afc->extract          = fr_command_iso_extract;
+	afc->get_mime_types   = fr_command_iso_get_mime_types;
+	afc->get_capabilities = fr_command_iso_get_capabilities;
 }
 
 
@@ -234,12 +244,9 @@ fr_command_iso_init (FrCommand *comm)
 {
 	FrCommandIso *comm_iso = FR_COMMAND_ISO (comm);
 
-	comm->file_type = FR_FILE_TYPE_ISO;
-
 	comm_iso->cur_path = NULL;
 	comm_iso->joliet = TRUE;
 
-	comm->propCanModify                = FALSE;
 	comm->propAddCanUpdate             = FALSE;
 	comm->propAddCanReplace            = FALSE;
 	comm->propExtractCanAvoidOverwrite = FALSE;
@@ -296,20 +303,3 @@ fr_command_iso_get_type ()
 
 	return type;
 }
-
-
-FrCommand *
-fr_command_iso_new (FrProcess  *process,
-		    const char *filename)
-{
-	FrCommand *comm;
-
-	if (! is_program_in_path ("isoinfo")) {
-		return NULL;
-	}
-
-	comm = FR_COMMAND (g_object_new (FR_TYPE_COMMAND_ISO, NULL));
-	fr_command_construct (comm, process, filename);
-
-	return comm;
-}
