@@ -30,6 +30,7 @@
 
 #include "file-data.h"
 #include "file-utils.h"
+#include "gio-utils.h"
 #include "glib-utils.h"
 #include "fr-command.h"
 #include "fr-command-rar.h"
@@ -107,7 +108,7 @@ process_line (char     *line,
 			rar_comm->odd_line = TRUE;
 		}
 		else if (strncmp (line, "Volume ", 7) == 0)
-			rar_comm->multi_volume = TRUE;
+			comm->multi_volume = TRUE;
 		return;
 	}
 
@@ -204,11 +205,119 @@ add_password_arg (FrCommand  *comm,
 }
 
 
+typedef enum {
+	FIRST_VOLUME_IS_000,
+	FIRST_VOLUME_IS_001,
+	FIRST_VOLUME_IS_RAR
+} FirstVolumeExtension;
+
+
+static char *
+get_first_volume_name (const char           *name,
+		       const char           *pattern,
+		       FirstVolumeExtension  extension_type)
+{
+	char   *volume_name = NULL;
+	GRegex *re;
+
+	re = g_regex_new (pattern, G_REGEX_CASELESS, 0, NULL);
+	if (g_regex_match (re, name, 0, NULL)) {
+		char **parts;
+		int    l, i;
+
+		parts = g_regex_split (re, name, 0);
+		l = strlen (parts[2]);
+		switch (extension_type) {
+		case FIRST_VOLUME_IS_000:
+			for (i = 0; i < l; i++)
+				parts[2][i] = '0';
+			break;
+
+		case FIRST_VOLUME_IS_001:
+			for (i = 0; i < l; i++)
+				parts[2][i] = (i < l - 1) ? '0' : '1';
+			break;
+
+		case FIRST_VOLUME_IS_RAR:
+			if (g_str_has_suffix (parts[1], "r")) {
+				parts[2][0] = 'a';
+				parts[2][1] = 'r';
+			}
+			else {
+				parts[2][0] = 'A';
+				parts[2][1] = 'R';
+			}
+			break;
+		}
+
+		volume_name = g_strjoinv ("", parts);
+
+		g_strfreev (parts);
+	}
+	g_regex_unref (re);
+
+	if (volume_name != NULL) {
+		char *tmp;
+
+		tmp = volume_name;
+		volume_name = g_filename_from_utf8 (tmp, -1, NULL, NULL, NULL);
+		g_free (tmp);
+	}
+
+	return volume_name;
+}
+
+
+static void
+check_multi_vomule (FrCommand *comm)
+{
+	GFile *file;
+	char   buffer[11];
+
+	file = g_file_new_for_path (comm->filename);
+	if (! g_load_file_in_buffer (file, buffer, 11, NULL)) {
+		g_object_unref (file);
+		return;
+	}
+
+	if ((buffer[10] & 0x01) == 0x01) {
+		char   *volume_name = NULL;
+		char   *name;
+
+		name = g_filename_to_utf8 (file_name_from_path (comm->filename), -1, NULL, NULL, NULL);
+
+		volume_name = get_first_volume_name (name, "^(.*\\.part)([0-9]+)(\\.rar)$", FIRST_VOLUME_IS_001);
+		if (volume_name == NULL)
+			volume_name = get_first_volume_name (name, "^(.*\\.r)([0-9]+)$", FIRST_VOLUME_IS_RAR);
+		if (volume_name == NULL)
+			volume_name = get_first_volume_name (name, "^(.*\\.)([0-9]+)$", FIRST_VOLUME_IS_001);
+
+		if (volume_name != NULL) {
+			GFile *parent;
+			GFile *child;
+			char  *volume_filename;
+
+			parent = g_file_get_parent (file);
+			child = g_file_get_child (parent, volume_name);
+			volume_filename = g_file_get_path (child);
+			fr_command_set_multi_volume (comm, volume_filename);
+
+			g_free (volume_filename);
+			g_object_unref (child);
+			g_object_unref (parent);
+		}
+
+		g_free (name);
+	}
+
+	g_object_unref (file);
+}
+
+
 static void
 fr_command_rar_list (FrCommand  *comm)
 {
-	FR_COMMAND_RAR (comm)->list_started = FALSE;
-	FR_COMMAND_RAR (comm)->multi_volume = FALSE;
+	check_multi_vomule (comm);
 
 	fr_process_set_out_line_func (FR_COMMAND (comm)->process,
 				      process_line,
@@ -229,6 +338,8 @@ fr_command_rar_list (FrCommand  *comm)
 
 	fr_process_add_arg (comm->process, comm->filename);
 	fr_process_end_command (comm->process);
+
+	FR_COMMAND_RAR (comm)->list_started = FALSE;
 	fr_process_start (comm->process);
 }
 
@@ -247,17 +358,27 @@ parse_progress_line (FrCommand  *comm,
 
 	prefix_len = strlen (prefix);
 	if (strncmp (line, prefix, prefix_len) == 0) {
-		double fraction;
-		int    len;
+		double  fraction;
+		int     len;
+		char   *b_idx;
 
 		strcpy (Progress_Filename, line + prefix_len);
+
+		/* when a new volume is created a sequence of backspaces is
+		 * issued, remove the backspaces from the filename */
+		b_idx = strchr (Progress_Filename, '\x08');
+		if (b_idx != NULL)
+			*b_idx = 0;
+
+		/* remove the OK at the end of the filename */
 		len = strlen (Progress_Filename);
 		if ((len > 5) && (strncmp (Progress_Filename + len - 5, "  OK ", 5) == 0))
 			Progress_Filename[len - 5] = 0;
+
 		sprintf (Progress_Message, "%s%s", message_prefix, file_name_from_path (Progress_Filename));
 		fr_command_message (comm, Progress_Message);
 
-		fraction= (double) ++comm->n_file / (comm->n_files + 1);
+		fraction = (double) ++comm->n_file / (comm->n_files + 1);
 		fr_command_progress (comm, fraction);
 	}
 }
@@ -270,12 +391,23 @@ process_line__add (char     *line,
 	FrCommand *comm = FR_COMMAND (data);
 
 	if (strncmp (line, "Creating archive ", 17) == 0) {
+		const char *archive_filename = line + 17;
 		char *uri;
 
-		uri = g_filename_to_uri (line + 17, NULL, NULL);
-		fr_command_working_archive (comm, uri);
-		g_free (uri);
+		uri = g_filename_to_uri (archive_filename, NULL, NULL);
+		if ((comm->volume_size > 0)
+		    && g_regex_match_simple ("^.*\\.part(0)*2\\.rar$", uri, G_REGEX_CASELESS, 0))
+		{
+			char *volume_filename;
 
+			volume_filename = g_strdup (archive_filename);
+			volume_filename[strlen (volume_filename) - 5] = '1';
+			fr_command_set_multi_volume (comm, volume_filename);
+			g_free (volume_filename);
+		}
+		fr_command_working_archive (comm, uri);
+
+		g_free (uri);
 		return;
 	}
 
@@ -551,8 +683,7 @@ FrCommandCap
 fr_command_rar_get_capabilities (FrCommand  *comm,
 			         const char *mime_type)
 {
-	FrCommandRar *comm_rar = FR_COMMAND_RAR (comm);
-	FrCommandCap  capabilities;
+	FrCommandCap capabilities;
 
 	capabilities = FR_COMMAND_CAN_ARCHIVE_MANY_FILES | FR_COMMAND_CAN_ENCRYPT | FR_COMMAND_CAN_ENCRYPT_HEADER;
 	if (is_program_in_path ("rar"))
@@ -561,7 +692,7 @@ fr_command_rar_get_capabilities (FrCommand  *comm,
 		capabilities |= FR_COMMAND_CAN_READ;
 
 	/* multi-volumes are read-only */
-	if ((comm->files->len > 0) && comm_rar->multi_volume && (capabilities & FR_COMMAND_CAN_WRITE))
+	if ((comm->files->len > 0) && comm->multi_volume && (capabilities & FR_COMMAND_CAN_WRITE))
 		capabilities ^= FR_COMMAND_CAN_WRITE;
 
 	return capabilities;
